@@ -23,7 +23,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -52,10 +52,10 @@ extern int nand_correct_data(struct nand_device *nand, u_char *dat,
 #define SPARE_OFFS 0x140
 #define DATA_OFFS  0x200
 
-static int sp_ooblayout[] = {
+static const int sp_ooblayout[] = {
 	10, 11, 12, 13, 14, 15
 };
-static int lp_ooblayout[] = {
+static const int lp_ooblayout[] = {
 	40, 41, 42, 43, 44, 45,
 	46, 47, 48, 49, 50, 51,
 	52, 53, 54, 55, 56, 57,
@@ -1040,8 +1040,8 @@ static int lpc32xx_write_page_slc(struct nand_device *nand,
 	int retval;
 	uint32_t target_mem_base;
 
-	LOG_DEBUG("SLC write page %x data=%d, oob=%d, "
-		"data_size=%d, oob_size=%d",
+	LOG_DEBUG("SLC write page %" PRIx32 " data=%d, oob=%d, "
+		"data_size=%" PRIu32 ", oob_size=%" PRIu32,
 		page, data != 0, oob != 0, data_size, oob_size);
 
 	target_mem_base = pworking_area->address;
@@ -1148,7 +1148,7 @@ static int lpc32xx_write_page_slc(struct nand_device *nand,
 		/* Copy to oob, at correct offsets */
 		static uint8_t ecc[24];
 		slc_ecc_copy_to_buffer(ecc, hw_ecc, ecc_count);
-		int *layout = nand->page_size == 2048 ? lp_ooblayout : sp_ooblayout;
+		const int *layout = nand->page_size == 2048 ? lp_ooblayout : sp_ooblayout;
 		int i;
 		for (i = 0; i < ecc_count * 3; i++)
 			foob[layout[i]] = ecc[i];
@@ -1451,7 +1451,7 @@ static int lpc32xx_read_page_slc(struct nand_device *nand,
 	int retval;
 	uint32_t target_mem_base;
 
-	LOG_DEBUG("SLC read page %x data=%d, oob=%d",
+	LOG_DEBUG("SLC read page %" PRIx32 " data=%" PRIu32 ", oob=%" PRIu32,
 		page, data_size, oob_size);
 
 	target_mem_base = pworking_area->address;
@@ -1549,7 +1549,7 @@ static int lpc32xx_read_page_slc(struct nand_device *nand,
 	slc_ecc_copy_to_buffer(ecc, hw_ecc, ecc_count);
 	/* Copy ECC from flash using correct layout */
 	static uint8_t fecc[24];/* max size */
-	int *layout = nand->page_size == 2048 ? lp_ooblayout : sp_ooblayout;
+	const int *layout = nand->page_size == 2048 ? lp_ooblayout : sp_ooblayout;
 	int i;
 	for (i = 0; i < ecc_count * 3; i++)
 		fecc[i] = foob[layout[i]];
@@ -1558,7 +1558,7 @@ static int lpc32xx_read_page_slc(struct nand_device *nand,
 		retval = nand_correct_data(nand, data + 256*i, &fecc[i * 3],
 				&ecc[i * 3]);
 		if (retval > 0)
-			LOG_WARNING("error detected and corrected: %d/%d",
+			LOG_WARNING("error detected and corrected: %" PRIu32 "/%d",
 				page, i);
 		if (retval < 0)
 			break;
@@ -1566,7 +1566,7 @@ static int lpc32xx_read_page_slc(struct nand_device *nand,
 	if (i == ecc_count)
 		retval = ERROR_OK;
 	else {
-		LOG_ERROR("uncorrectable error detected: %d/%d", page, i);
+		LOG_ERROR("uncorrectable error detected: %" PRIu32 "/%d", page, i);
 		retval = ERROR_NAND_OPERATION_FAILED;
 	}
 	return retval;
