@@ -15,7 +15,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
  ***************************************************************************/
 
 #ifndef CFI_H
@@ -143,8 +143,8 @@ struct cfi_unlock_addresses {
 struct cfi_fixup {
 	uint16_t mfr;
 	uint16_t id;
-	void (*fixup)(struct flash_bank *bank, void *param);
-	void *param;
+	void (*fixup)(struct flash_bank *bank, const void *param);
+	const void *param;
 };
 
 #define CFI_MFR_AMD		0x0001
