@@ -18,7 +18,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
  ***************************************************************************/
 
 #ifndef FLASH_NOR_DRIVER_H
@@ -137,7 +137,7 @@ struct flash_driver {
 	 * @returns ERROR_OK if successful; otherwise, an error code.
 	 */
 	int (*write)(struct flash_bank *bank,
-			uint8_t *buffer, uint32_t offset, uint32_t count);
+			const uint8_t *buffer, uint32_t offset, uint32_t count);
 
 	/**
 	 * Read data from the flash. Note CPU address will be
