@@ -14,7 +14,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -29,6 +29,8 @@ extern struct flash_driver lpcspifi_flash;
 extern struct flash_driver cfi_flash;
 extern struct flash_driver at91sam3_flash;
 extern struct flash_driver at91sam4_flash;
+extern struct flash_driver at91sam4l_flash;
+extern struct flash_driver at91samd_flash;
 extern struct flash_driver at91sam7_flash;
 extern struct flash_driver str7x_flash;
 extern struct flash_driver str9x_flash;
@@ -50,6 +52,13 @@ extern struct flash_driver dsp5680xx_flash;
 extern struct flash_driver fm3_flash;
 extern struct flash_driver kinetis_flash;
 extern struct flash_driver efm32_flash;
+extern struct flash_driver mdr_flash;
+extern struct flash_driver mini51_flash;
+extern struct flash_driver nuc1x_flash;
+extern struct flash_driver nrf51_flash;
+extern struct flash_driver mrvlqspi_flash;
+extern struct flash_driver psoc4_flash;
+extern struct flash_driver sim3x_flash;
 
 /**
  * The list of built-in flash drivers.
@@ -64,6 +73,8 @@ static struct flash_driver *flash_drivers[] = {
 	&at91sam7_flash,
 	&at91sam3_flash,
 	&at91sam4_flash,
+	&at91sam4l_flash,
+	&at91samd_flash,
 	&str7x_flash,
 	&str9x_flash,
 	&aduc702x_flash,
@@ -84,6 +95,13 @@ static struct flash_driver *flash_drivers[] = {
 	&dsp5680xx_flash,
 	&kinetis_flash,
 	&efm32_flash,
+	&mdr_flash,
+	&mini51_flash,
+	&nuc1x_flash,
+	&nrf51_flash,
+	&mrvlqspi_flash,
+	&psoc4_flash,
+	&sim3x_flash,
 	NULL,
 };
 
