@@ -21,7 +21,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -33,7 +33,7 @@
 #include "fileio.h"
 
 struct fileio_internal {
-	const char *url;
+	char *url;
 	ssize_t size;
 	enum fileio_type type;
 	enum fileio_access access;
@@ -141,7 +141,7 @@ int fileio_close(struct fileio *fileio_p)
 
 	retval = fileio_close_local(fileio);
 
-	free((void *)fileio->url);
+	free(fileio->url);
 	fileio->url = NULL;
 
 	free(fileio);
