@@ -24,7 +24,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
  ***************************************************************************/
 
 #ifndef ARM7_9_COMMON_H
@@ -118,6 +118,20 @@ struct arm7_9_common {
 
 	void (*pre_restore_context)(struct target *target);
 	/**< Callback function called before restoring the processor context */
+
+	/**
+	 * Variant specific memory write function that does not dispatch to bulk_write_memory.
+	 * Used as a fallback when bulk writes are unavailable, or for writing data needed to
+	 * do the bulk writes.
+	 */
+	int (*write_memory)(struct target *target, uint32_t address,
+			uint32_t size, uint32_t count, const uint8_t *buffer);
+	/**
+	 * Write target memory in multiples of 4 bytes, optimized for
+	 * writing large quantities of data.
+	 */
+	int (*bulk_write_memory)(struct target *target, uint32_t address,
+			uint32_t count, const uint8_t *buffer);
 };
 
 static inline struct arm7_9_common *target_to_arm7_9(struct target *target)
@@ -151,6 +165,10 @@ int arm7_9_read_memory(struct target *target, uint32_t address,
 		uint32_t size, uint32_t count, uint8_t *buffer);
 int arm7_9_write_memory(struct target *target, uint32_t address,
 		uint32_t size, uint32_t count, const uint8_t *buffer);
+int arm7_9_write_memory_opt(struct target *target, uint32_t address,
+		uint32_t size, uint32_t count, const uint8_t *buffer);
+int arm7_9_write_memory_no_opt(struct target *target, uint32_t address,
+		uint32_t size, uint32_t count, const uint8_t *buffer);
 int arm7_9_bulk_write_memory(struct target *target, uint32_t address,
 		uint32_t count, const uint8_t *buffer);
 
