@@ -16,7 +16,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -823,6 +823,7 @@ static int arm920t_init_arch_info(struct target *target,
 
 	arm7_9->post_debug_entry = arm920t_post_debug_entry;
 	arm7_9->pre_restore_context = arm920t_pre_restore_context;
+	arm7_9->write_memory = arm920t_write_memory;
 
 	arm920t->armv4_5_mmu.armv4_5_cache.ctype = -1;
 	arm920t->armv4_5_mmu.get_ttb = arm920t_get_ttb;
@@ -1697,14 +1698,12 @@ struct target_type arm920t_target = {
 	.get_gdb_reg_list = arm_get_gdb_reg_list,
 
 	.read_memory = arm920t_read_memory,
-	.write_memory = arm920t_write_memory,
+	.write_memory = arm7_9_write_memory_opt,
 	.read_phys_memory = arm920t_read_phys_memory,
 	.write_phys_memory = arm920t_write_phys_memory,
 	.mmu = arm920_mmu,
 	.virt2phys = arm920_virt2phys,
 
-	.bulk_write_memory = arm7_9_bulk_write_memory,
-
 	.checksum_memory = arm_checksum_memory,
 	.blank_check_memory = arm_blank_check_memory,
 
