@@ -15,7 +15,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
  ***************************************************************************/
 
 #ifndef DSP563XX_H
@@ -31,6 +31,16 @@ struct mcu_jtag {
 	struct jtag_tap *tap;
 };
 
+enum breakpoint_usage {
+	BPU_NONE = 0,
+	BPU_BREAKPOINT,
+	BPU_WATCHPOINT
+};
+
+struct hardware_breakpoint {
+	enum breakpoint_usage used;
+};
+
 struct dsp563xx_common {
 	struct mcu_jtag jtag_info;
 	struct reg_cache *core_cache;
@@ -40,6 +50,11 @@ struct dsp563xx_common {
 	/* register cache to processor synchronization */
 	int (*read_core_reg) (struct target *target, int num);
 	int (*write_core_reg) (struct target *target, int num);
+
+	struct hardware_breakpoint hardware_breakpoint[1];
+
+	/*Were the hardware breakpoints cleared on startup?*/
+	int hardware_breakpoints_cleared;
 };
 
 struct dsp563xx_core_reg {
