@@ -15,7 +15,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
  ***************************************************************************/
 
 #ifndef DSP563XX_ONCE_H
@@ -23,6 +23,10 @@
 
 #include <jtag/jtag.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #define DSP563XX_ONCE_OCR_EX	(1<<5)
 #define DSP563XX_ONCE_OCR_GO	(1<<6)
 #define DSP563XX_ONCE_OCR_RW	(1<<7)
@@ -59,9 +63,9 @@
 #define DSP563XX_ONCE_NOREG	0x01F	/* no register selected */
 
 struct once_reg {
-	uint8_t num;
-	uint8_t addr;
-	uint8_t len;
+	const uint8_t num;
+	const uint8_t addr;
+	const uint8_t len;
 	const char *name;
 	uint32_t reg;
 };
