@@ -15,7 +15,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -27,7 +27,7 @@
 #include "etb.h"
 #include "register.h"
 
-static char *etb_reg_list[] = {
+static const char * const etb_reg_list[] = {
 	"ETB_identification",
 	"ETB_ram_depth",
 	"ETB_ram_width",
@@ -165,7 +165,6 @@ static void etb_getbuf(jtag_callback_data_t arg)
 	*((uint32_t *)arg) = buf_get_u32(in, 0, 32);
 }
 
-
 static int etb_read_ram(struct etb *etb, uint32_t *data, int num_frames)
 {
 	struct scan_field fields[3];
@@ -193,7 +192,7 @@ static int etb_read_ram(struct etb *etb, uint32_t *data, int num_frames)
 	jtag_add_dr_scan(etb->tap, 3, fields, TAP_IDLE);
 
 	for (i = 0; i < num_frames; i++) {
-		/* ensure nR/W reamins set to read */
+		/* ensure nR/W remains set to read */
 		buf_set_u32(&temp2, 0, 1, 0);
 
 		/* address remains set to 0x4 (RAM data) until we read the last frame */
@@ -309,7 +308,7 @@ static int etb_write_reg(struct reg *reg, uint32_t value)
 	fields[0].num_bits = 32;
 	uint8_t temp0[4];
 	fields[0].out_value = temp0;
-	buf_set_u32(&temp0, 0, 32, value);
+	buf_set_u32(temp0, 0, 32, value);
 	fields[0].in_value = NULL;
 
 	fields[1].num_bits = 7;
@@ -442,7 +441,7 @@ static const struct command_registration etb_command_handlers[] = {
 	{
 		.name = "etb",
 		.mode = COMMAND_ANY,
-		.help = "Emebdded Trace Buffer command group",
+		.help = "Embedded Trace Buffer command group",
 		.chain = etb_config_command_handlers,
 	},
 	COMMAND_REGISTRATION_DONE
