@@ -18,7 +18,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
  ***************************************************************************/
 
 #ifndef ETM_H
@@ -72,7 +72,7 @@ enum {
 };
 
 struct etm_reg {
-	uint32_t value;
+	uint8_t value[4];
 	const struct etm_reg_info *reg_info;
 	struct arm_jtag *jtag_info;
 };
