@@ -21,7 +21,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
  ***************************************************************************/
 
 #ifndef TARGET_TYPE_H
@@ -101,7 +101,8 @@ struct target_type {
 	 * list, however it is after GDB is connected that monitor commands can
 	 * be run to properly initialize the target
 	 */
-	int (*get_gdb_reg_list)(struct target *target, struct reg **reg_list[], int *reg_list_size);
+	int (*get_gdb_reg_list)(struct target *target, struct reg **reg_list[],
+			int *reg_list_size, enum target_register_class reg_class);
 
 	/* target memory access
 	* size: 1 = byte (8bit), 2 = half-word (16bit), 4 = word (32bit)
@@ -129,14 +130,6 @@ struct target_type {
 	int (*write_buffer)(struct target *target, uint32_t address,
 			uint32_t size, const uint8_t *buffer);
 
-	/**
-	 * Write target memory in multiples of 4 bytes, optimized for
-	 * writing large quantities of data.  Do @b not call this
-	 * function directly, use target_bulk_write_memory() instead.
-	 */
-	int (*bulk_write_memory)(struct target *target, uint32_t address,
-			uint32_t count, const uint8_t *buffer);
-
 	int (*checksum_memory)(struct target *target, uint32_t address,
 			uint32_t count, uint32_t *checksum);
 	int (*blank_check_memory)(struct target *target, uint32_t address,
@@ -173,6 +166,11 @@ struct target_type {
 	 */
 	int (*remove_watchpoint)(struct target *target, struct watchpoint *watchpoint);
 
+	/* Find out just hit watchpoint. After the target hits a watchpoint, the
+	 * information could assist gdb to locate where the modified/accessed memory is.
+	 */
+	int (*hit_watchpoint)(struct target *target, struct watchpoint **hit_watchpoint);
+
 	/**
 	 * Target algorithm support.  Do @b not call this method directly,
 	 * use target_run_algorithm() instead.
@@ -227,6 +225,13 @@ struct target_type {
 	 * */
 	int (*init_target)(struct command_context *cmd_ctx, struct target *target);
 
+	/**
+	 * Free all the resources allocated by the target.
+	 *
+	 * @param target The target to deinit
+	 */
+	void (*deinit_target)(struct target *target);
+
 	/* translate from virtual to physical address. Default implementation is successful
 	 * no-op(i.e. virtual==physical).
 	 */
@@ -258,6 +263,19 @@ struct target_type {
 	 * circumstances.
 	 */
 	int (*check_reset)(struct target *target);
+
+	/* get GDB file-I/O parameters from target
+	 */
+	int (*get_gdb_fileio_info)(struct target *target, struct gdb_fileio_info *fileio_info);
+
+	/* pass GDB file-I/O response to target
+	 */
+	int (*gdb_fileio_end)(struct target *target, int retcode, int fileio_errno, bool ctrl_c);
+
+	/* do target profiling
+	 */
+	int (*profiling)(struct target *target, uint32_t *samples,
+			uint32_t max_num_samples, uint32_t *num_samples, uint32_t seconds);
 };
 
 #endif /* TARGET_TYPE_H */
